<?php
/***************************************************************************
*                                                                          *
*   (c) 2004 Vladimir V. Kalynyak, Alexey V. Vinokurov, Ilya M. Shalnev    *
*                                                                          *
* This  is  commercial  software,  only  users  who have purchased a valid *
* license  and  accept  to the terms of the  License Agreement can install *
* and use this program.                                                    *
*                                                                          *
****************************************************************************
* PLEASE READ THE FULL TEXT  OF THE SOFTWARE  LICENSE   AGREEMENT  IN  THE *
* "copyright.txt" FILE PROVIDED WITH THIS DISTRIBUTION PACKAGE.            *
****************************************************************************/

use Tygh\Http;
use Tygh\Registry;

/**
 * @var array $processor_data
 * @var array $order_info
 * @var string $mode
 */

require_once dirname(__FILE__) . "/../QuickPay/QuickPay.php";
use QuickPay\QuickPay;

if (!defined('BOOTSTRAP')) { die('Access denied'); }

// Return from quickpay website
if (defined('PAYMENT_NOTIFICATION')) {
    if ($mode == 'return') {
        //file_put_contents('_quickpay_return', print_r($_REQUEST,true));
        if (fn_check_payment_script('quickpay.php.txt', $_REQUEST['order_id'])) {
            $order_info = fn_get_order_info($_REQUEST['order_id'], true);

            if ($order_info['status'] == 'O') {
                $edp_data = fn_generate_ekeys_for_edp(array('status_from' => STATUS_INCOMPLETED_ORDER, 'status_to' => 'O'), $order_info);
                fn_order_notification($order_info, $edp_data);
            }

            if (fn_allowed_for('MULTIVENDOR')) {
                if ($order_info['status'] == STATUS_PARENT_ORDER) {
                    $child_orders = db_get_hash_single_array("SELECT order_id, status FROM ?:orders WHERE parent_order_id = ?i", array('order_id', 'status'), $_REQUEST['order_id']);
                    foreach ($child_orders as $order_id => $order_status) {
                        if ($order_status == 'O') {
                            $order_info = fn_get_order_info($order_id, true);
                            $edp_data = fn_generate_ekeys_for_edp(array('status_from' => STATUS_INCOMPLETED_ORDER, 'status_to' => 'O'), $order_info);
                            fn_order_notification($order_info, $edp_data);
                        }
                    }
                }
            }
        }
        fn_order_placement_routines('route', $_REQUEST['order_id'], false);

    } elseif ($mode == 'cancel') {
        $order_info = fn_get_order_info($_REQUEST['order_id']);

        //file_put_contents('_quickpay_cancel.txt', print_r($_REQUEST,true));
        $pp_response['order_status'] = 'N';
        $pp_response["reason_text"] = __('text_transaction_cancelled');

        fn_finish_payment($_REQUEST['order_id'], $pp_response);
        fn_order_placement_routines('route', $_REQUEST['order_id']);
        
    } 
} else {
    
    $merchant_id = $processor_data['processor_params']['merchant_id'];
    $private_key = $processor_data['processor_params']['private_key'];
    $agreement_id = $processor_data['processor_params']['api_user_agreement_id'];
    $api_key = $processor_data['processor_params']['api_user_key'];
    
    $quickpay_currency = $processor_data['processor_params']['currency'];
    //$quickpay_order_id = $processor_data['processor_params']['order_prefix'].(($order_info['repaid']) ? ($order_id .'_'. $order_info['repaid']) : $order_id);
    $quickpay_order_id = 'OrderID ' . (($order_info['repaid']) ? ($order_id .'_'. $order_info['repaid']) : $order_id);
    
    $payment_methods = array();
    foreach ( $processor_data['processor_params']['payment_methods'] as $payment => $value) {
        if ( $value == 'Y' ) {
            $payment_methods[] = $payment;
        }
        else {
            if ($payment != 'creditcard') {
                $payment_methods[] = '!' . $payment;
            }
        }
    }
    $payment_methods = implode(',', $payment_methods);
    //file_put_contents('_quickpay_payment_methods.txt', print_r($payment_methods,true));
    //file_put_contents('_quickpay_payment_methods_data.txt', print_r($processor_data['processor_params']['payment_methods'],true));
    $processor_data['processor_params']['autofee'] == 'Y' ? $autofee = 1 : $autofee = 0;
    
    if ($quickpay_currency == CART_PRIMARY_CURRENCY) {
        //Order Total
        $quickpay_total = fn_format_price($order_info['total'], $quickpay_currency);
    } else {
        $quickpay_total = fn_format_price_by_currency($order_info['total'], CART_PRIMARY_CURRENCY, $quickpay_currency);
    }

    $_phone = $order_info['phone'];

    $quickpay_url = "https://payment.quickpay.net";
    
    $return_url = fn_url("payment_notification.return?payment=quickpay&order_id=$order_id", AREA, 'current');
    $cancel_url = fn_url("payment_notification.cancel?payment=quickpay&order_id=$order_id", AREA, 'current');
    $notify_url = fn_url("payment_notification.quickpay_ipn", AREA, 'current');

    $post_data = array(
        'version' => "v10",
        'merchant_id' => $merchant_id,
        'agreement_id' => $agreement_id,
        'order_id' => $quickpay_order_id,
        'amount' => $quickpay_total * 100,
        'currency' => $quickpay_currency,
        
        'continueurl' => $return_url,
        'cancelurl' => $cancel_url,
        'callbackurl' => $notify_url,
        
        'type' => "payment",
        'autofee' => $autofee,
        'payment_methods' => $payment_methods,
        'customer_email' => $order_info['email'],
    );
    
    $invoice_address = array(
        'name'            => $order_info['b_firstname'] . ' ' . $order_info['b_lastname'],
        'att'             => empty($order_info['b_att']) ? '' : $order_info['b_att'],
        'street'          => empty($order_info['b_street']) ? '' : $order_info['b_street'],
        'house_number'    => empty($order_info['b_house_number']) ? '' : $order_info['b_house_number'],
        'house_extension' => empty($order_info['b_house_extension']) ? '' : $order_info['b_house_extension'],
        'zip_code'        => empty($order_info['b_zipcode']) ? '' : $order_info['b_zipcode'],
        'city'            => empty($order_info['b_city']) ? '' : $order_info['b_city'],
        'region'          => empty($order_info['b_region']) ? $order_info['b_state'] : $order_info['b_region'],
        'country_code'    => fn_get_iso_3_code($order_info['b_country']),
        'vat_no'          => empty($order_info['b_vat_no']) ? '' : $order_info['b_vat_no'],
        'phone_number'    => empty($order_info['b_phone_number'])  ? $_phone : $order_info['b_phone_number'],
        'mobile_number'   => empty($order_info['b_mobile_number']) ? $_phone : $order_info['b_mobile_number'],
        'email'           => $order_info['email'],
    );
    
    $shipping_address = array(
        'name'            => $order_info['s_firstname'] . ' ' . $order_info['s_lastname'],
        'att'             => empty($order_info['s_att']) ? '' : $order_info['s_att'],
        'street'          => empty($order_info['s_street']) ? '' : $order_info['s_street'],
        'house_number'    => empty($order_info['s_house_number']) ? '' : $order_info['s_house_number'],
        'house_extension' => empty($order_info['s_house_extension']) ? '' : $order_info['s_house_extension'],
        'zip_code'        => empty($order_info['s_zipcode']) ? '' : $order_info['s_zipcode'],
        'city'            => empty($order_info['s_city']) ? '' : $order_info['s_city'],
        'region'          => empty($order_info['s_region']) ? $order_info['s_state'] : $order_info['s_region'],
        'country_code'    => fn_get_iso_3_code($order_info['s_country']),
        'vat_no'          => empty($order_info['s_vat_no']) ? '' : $order_info['s_vat_no'],
        'phone_number'    => empty($order_info['s_phone_number'])  ? $_phone : $order_info['s_phone_number'],
        'mobile_number'   => empty($order_info['s_mobile_number']) ? $_phone : $order_info['s_mobile_number'],
        'email'           => $order_info['email'],
    );
    
    $basket = array();
    $vat_rate = 0;
    foreach ($order_info['taxes'] as $tax) {
        if ($tax['price_includes_tax'] == 'Y' && $tax['rate_type'] == 'P') {
            $vat_rate += round($tax['rate_value'], 2) /100;
        }
    }
    $x = $order_info['taxes'];
    foreach ($order_info['products'] as $product ) {
        $basket[] = array(
            'qty'        => $product['amount'],
            'item_no'    => $product['product_id'],
            'item_name'  => $product['product_code'],
            'item_price' => fn_format_price_by_currency($product['price'], CART_PRIMARY_CURRENCY, $quickpay_currency) * 100,
            'vat_rate'   => $vat_rate,
        );
    }
    
    $post_data['invoice_address'] = $invoice_address;
    $post_data['shipping_address'] = $shipping_address;
    //$post_data['basket'] = $basket;
    //$post_data["checksum"] = fn_quickpay_sign($post_data, $api_key);

    //file_put_contents('_quickpay_form_data.txt', print_r($post_data,true));
    //file_put_contents('_quickpay_order.txt', print_r($order_info,true));
    /*
    list($products, $product_count) = fn_pp_standart_prepare_products($order_info, $paypal_currency);
    $post_data = array_merge($post_data, $products);
    */
    
    // use API
    /*
    $payment_info = db_get_field("SELECT data FROM ?:order_data WHERE order_id = ?i AND type = 'P'", $order_id);
    if (!empty($payment_info)) {
        $payment_info = unserialize(fn_decrypt_text($payment_info));
    } else {
        $payment_info = array();
    }
    */
    
    $client = new QuickPay(":$api_key");
    $payment_id = fn_get_quickpay_payment_by_order_id($order_id);
    
    if (empty($payment_id)) {
        $payment = $client->request->post("/payments", $post_data);
        $payment_data = $payment->asArray();
        $payment_id = $payment_data['id'];
        if (!empty($payment_id) && !empty($order_id)) {
            db_query('INSERT INTO ?:quickpay_payments ?e', array('order_id' => $order_id, 'payment_id' => $payment_id));
        }
    }
    else {
        $payment = $client->request->patch("/payments/$payment_id", $post_data);
        $payment_data = $payment->asArray();
        if (!empty($payment_id) && !empty($order_id)) {
            db_query("UPDATE ?:quickpay_payments SET ?u WHERE order_id = ?i", array('payment_id' => $payment_id), $order_id);
        }
    }
    //file_put_contents('_quickpay_payment_api.txt', print_r($payment_data,true));
    
    $status = $payment->httpStatus();
    if ($status == 200 || $status == 201) {
        $link = $client->request->put("/payments/$payment_id/link", $post_data);
        $link_data = $link->asArray();
        //file_put_contents('_quickpay_link_1.txt', print_r($link,true));
        //file_put_contents('_quickpay_link_2.txt', print_r($link_data,true));
    }
    
    if ($order_info['status'] == STATUS_INCOMPLETED_ORDER) {
        fn_change_order_status($order_id, 'O', '', false);
    }
    if (fn_allowed_for('MULTIVENDOR')) {
        if ($order_info['status'] == STATUS_PARENT_ORDER) {
            $child_orders = db_get_hash_single_array("SELECT order_id, status FROM ?:orders WHERE parent_order_id = ?i", array('order_id', 'status'), $order_id);

            foreach ($child_orders as $order_id => $order_status) {
                if ($order_status == STATUS_INCOMPLETED_ORDER) {
                    fn_change_order_status($order_id, 'O', '', false);
                }
            }
        }
    }
    if ($order_info['payment_info']['quickpay_payment_type'] == 'link') {
        empty( $link_data['url'] ) ? fn_redirect(fn_url('checkout.checkout'), false) : fn_redirect($link_data['url'], true);
    }
    
    if ($order_info['payment_info']['quickpay_payment_type'] == 'card') {
        $card_id = fn_get_quickpay_card_by_user_id($order_info['user_id']);
        if(!empty($card_id)) {
            $tokens    =  $client->request->post("/cards/$card_id/tokens")->asArray();
            //file_put_contents('_tokens.txt', print_r($tokens,true));
            $authorize = $client->request->post("/payments/$payment_id/authorize?synchronized", array(
                'amount' => $post_data['amount'],
                'card' => array ( 'token' => $tokens['token'] )
            ));
            //file_put_contents('_authorize.txt', print_r($authorize->asArray(),true));
            fn_redirect($return_url, false);
        }
    }
    fn_redirect(fn_url('checkout.checkout'), false);
    //fn_create_payment_form($link_data['url'], array(), 'QuickPay Server', false);
}
exit;
